import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.logging.LogManager;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.hyades.logging.core.Guid;
import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.ComponentIdentification;
import org.eclipse.hyades.logging.events.cbe.EventFactory;
import org.eclipse.hyades.logging.events.cbe.ReportSituation;
import org.eclipse.hyades.logging.events.cbe.Situation;
import org.eclipse.hyades.logging.events.cbe.impl.EventFactoryContext;
import org.eclipse.hyades.logging.events.cbe.util.EventHelpers;
import org.eclipse.hyades.logging.java.CommonBaseEventLogRecord;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.MessageDialogWithToggle;
import org.eclipse.ui.IWorkbenchWindow;

import com.ibm.etools.common.logging.CommonLoggingPlugin;
import com.ibm.etools.common.logging.loggers.CommonBaseEventLog;
import com.ibm.etools.common.logging.loggers.ICommonBaseEventLog;
import com.ibm.etools.common.logging.records.CommonBaseEventStatus;
import com.ibm.etools.common.logging.records.ICommonBaseEventStatus;
import com.ibm.etools.common.logging.util.CommonLoggingConstants;
import com.ibm.etools.common.logging.util.Level;

/**********************************************************************
 * Licensed Material - Property of IBM
 * 
 * (C) Copyright IBM Corp. 2004, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 **********************************************************************/

/** 
 * Sample class to demonstrate the usage of Common Logging.
 * <p>
 * This sample covers the following scenarios:
 * <p>
 * <ol>
 * <li>Logging an <code>IStatus</code> log record with one child <code>IStatus</code> to the sample plug-in's <code>ILog</code>.</li>
 * <li>Logging an <code>ICommonBaseEventStatus</code> log record with a nested <code>CommonBaseEvent</code> to the sample plug-in's <code>ILog</code>.</li>
 * <li>Logging a <code>CommonBaseEvent</code> log record, an <code>IStatus</code> log record with one child <code>IStatus</code> and an <code>ICommonBaseEventStatus</code> log record with a nested <code>CommonBaseEvent</code> to the sample plug-in's <code>ICommonBaseEventLog</code>.</li>
 * <li>Logging a <code>CommonBaseEvent</code> log record based on the sample plug-in's default Common Base Event template to the sample plug-in's <code>ICommonBaseEventLog</code>.</li>
 * <li>Logging a <code>CommonBaseEventLogRecord</code> log record with a nested <code>CommonBaseEvent</code> to the Common Logging logger (<code>java.util.logging.Logger</code>).</li>
 * </ol>
 * <p>
 * All <code>IStatus</code> log records are converted to Common Base Events using the following mapping:
 * <p>
 * <table border="1" cellpadding="10">
 *	     <tr><th>IStatus Property</th><th>CommonBaseEvent Property</th></tr>
 *	     <tr><td>Code</td>            <td>ExtendedDataElement.Name = "Code"<br>
 *	                                      ExtendedDataElement.Type = "integer"<br>
 *	                                      ExtendedDataElement.Values[0] = &lt;Code&gt;</td></tr>
 *	     <tr><td>Exception</td>       <td>ExtendedDataElement.Name = &quot;Throwable&quot; <br>
 *	                                      ExtendedDataElement.Type = &quot;stringArray&quot; <br>
 *	                                      ExtendedDataElement.Values[0] = &lt;Throwable's class name&gt;[: &lt;Throwable's localized message&gt;] <br>
 *	                                      ExtendedDataElement.Values[1...n] = &lt;Throwable's stackTraceElement[0...(n - 1)]&gt; <br>
 *	                                      [ExtendedDataElement.Children[0] = &lt;Throwable's cause&gt;] <br>
 *	                                      <br>
 *	                                      where: <br>
 *	                                      <br>
 *	                                      ExtendedDataElement.Children[0].Name = &quot;Cause&quot; <br>
 *	                                      ExtendedDataElement.Children[0].Type = &quot;stringArray&quot; <br>
 *	                                      ExtendedDataElement.Children[0].Values[0] = &lt;cause's class name&gt;[: &lt;cause's localized message&gt;] <br>
 *	                                      ExtendedDataElement.Children[0].Values[1...n] = &lt;cause's stackTraceElement[0...(n - 1)]&gt; <br>
 *	                                      [ExtendedDataElement.Children[0].Children[0] = &lt;cause's cause&gt;]</td></tr>
 *	     <tr><td>Message</td>          <td>msg</td></tr>
 *	     <tr><td>Plug-in</td>          <td>SourceComponentId.Application = "Common Logging"<br>
 *	                          	          SourceComponentId.Location = &lt;local host's IPv4 IP address&gt; or "127.0.0.1"<br>
 *	                          	          SourceComponentId.LocationType = "IPV4"<br>
 *	                          	          SourceComponentId.Component = &lt;Plug-in&gt;<br>
 *	                          	          SourceComponentId.SubComponent = "Log"<br>
 *	                          	          SourceComponentId.ComponentIDType = "Application"<br>
 *	                          	          SourceComponentId.ComponetType = "Eclipse Plug-ins"</td></tr>
 *	     <tr><td>Severity</td>        <td>Severity = &lt;Severity&gt; (see mapping table)</td></tr>
 *	     <tr><td>Children[0...n]</td> <td>ExtendedDataElement[0...n].Name = &lt;Children&lt;n&gt;.Plugin&gt;<br>
 *	                          	          ExtendedDataElement[0...n].Type = "string"<br>
 *	                          	          ExtendedDataElement[0...n].Children[0...m] = &lt;Children&lt;n&gt;.Code&gt;<br>
 *	                          	          ExtendedDataElement[0...n].Children[0...m] = &lt;Children&lt;n&gt;.Exception&gt;<br>
 *	                          	          ExtendedDataElement[0...n].Children[0...m] = &lt;Children&lt;n&gt;.Message&gt;<br>
 *	                          	          ExtendedDataElement[0...n].Children[0...m] = &lt;Children&lt;n&gt;.Severity&gt;<br>
 *	                          	          ExtendedDataElement[0...n].Children[0...m] = &lt;Children&lt;n&gt;.Children&gt;</td></tr>
 * </table>
 * <p>
 * The mapping process automatically populates the following Common Base Event properties:
 * <p>
 * <table border="1" cellpadding="10">
 *	     <tr><th>CommonBaseEvent Property</th><th>Value</th></tr>
 *	     <tr><td>CreationTime</td>            <td>Current time in milliseconds</td></tr>
 *	     <tr><td>GlobalInstanceId</td>        <td>Unique GUID</td></tr>
 *	     <tr><td>Situation</td>               <td>Situation.CategoryName = "ReportSituation"<br>
 *                                                Situation.SituationType.ReasoningScope = "INTERNAL"<br>
 *                                                Situation.SituationType.ReportCategory = "LOG/TRACE"</td></tr>
 * </table>
 * <p>
 * The conversion process uses the following mapping to convert the severity of the <code>IStatus</code> 
 * log record to a Common Base Event severity:
 * <p>
 * <table border="1" cellpadding="10">
 *	     <tr><th>IStatus Severity</th><th>CommonBaseEvent Severity</th></tr>
 *	     <tr><td>OK</td>              <td>9</td></tr>
  *	     <tr><td>INFO</td>            <td>19</td></tr>
 *	     <tr><td>WARNING</td>         <td>49</td></tr>
 *	     <tr><td>ERROR</td>           <td>69</td></tr>
 *	     <tr><td>CANCEL</td>          <td>70</td></tr>
 * </table>
 * <p>
 * All logged records are potentially converted to Common Base Events, serialized to XML and written to the following locations:
 * <p>
 * <ol>
 * <li>&lt;workspace&gt;/.metadata/CommonBaseEvent.xml</li>
 * <li>The "Common.Logging.Logger" Logging Agent</li>
 * </ol>
 * <p>
 * where <code>&lt;workspace&gt;</code> is the absolute directory of the
 * Eclipse workspace.
 * <p>
 * NOTE: The Agent Controller service or daemon must be configured and running to utilize the Logging Agent functionality in this sample.
 * <p>
 * Furthermore, all logged records are potentially converted to <code>ICommonBaseEventStatus</code>s, serialized to plain text and written 
 * to the following location:
 * <p>
 * <ol>
 * <li>&lt;workspace&gt;/.metadata/.log</li>
 * </ol>
 * <p>
 * where <code>&lt;workspace&gt;</code> is the absolute directory of the
 * Eclipse workspace.
 * <p>
 * This class has the following dependencies:
 * <p>
 * <table border="1" cellpadding="10">
 *	     <tr><th>JAR file</th>     							<th>Plug-in it is contained in</th>          	<th>Reason for requirement</th></tr>
 *	     <tr><td>com.ibm.etools.common.logging.jar</td>  	<td>com.ibm.etools.common.logging</td>       	<td>Common Logging implementation</td></tr>
 *	     <tr><td>org.eclipse.core.runtime.jar</td>  		<td>org.eclipse.core.runtime</td>            	<td>Eclipse Logging implementation</td></tr>
 *	     <tr><td>hlcore.jar</td>   							<td>org.eclipse.hyades.logging.core</td>     	<td>Common Base Event's TPTP Logging Core dependency</td></tr>
 *	     <tr><td>hlcbe101.jar</td> 							<td>org.eclipse.hyades.logging.core</td>     	<td>Common Base Event implementation</td></tr>
 *	     <tr><td>hl14.jar</td>     							<td>org.eclipse.hyades.logging.core</td>   		<td>TPTP's support for JSR-047 logging APIs</td></tr>
 *	     <tr><td>org.eclipse.emf.ecore.jar</td>    			<td>org.eclipse.emf.ecore</td>               	<td>Common Base Event's Eclipse Modeling Framework (EMF) dependency</td></tr>
 *	     <tr><td>org.eclipse.emf.common.jar</td>  			<td>org.eclipse.emf.common</td>              	<td>Common Base Event's Eclipse Modeling Framework (EMF) dependency</td></tr>
 *	     <tr><td>hexr.jar</td>     							<td>org.eclipse.hyades.execution</td>		 	<td>TPTP's Logging Agent</td></tr>
 * </table>
 * <p>
 * 
 * @author  Paul E. Slauenwhite
 * @version July 13, 2006
 * @since   June 14, 2004
 * @see     org.eclipse.core.runtime.IStatus
 * @see     org.eclipse.core.runtime.MultiStatus
 * @see     org.eclipse.core.runtime.ILog
 * @see     org.eclipse.core.runtime.ILogListener
 * @see     org.eclipse.hyades.logging.events.cbe.CommonBaseEvent
 * @see     com.ibm.etools.common.logging.records.ICommonBaseEventStatus
 * @see     com.ibm.etools.common.logging.records.CommonBaseEventStatus
 * @see     com.ibm.etools.common.logging.loggers.ICommonBaseEventLog
 * @see     com.ibm.etools.common.logging.loggers.CommonBaseEventLog
 */
public class CommonLoggingSample {
    	
    /**
	 * IP address (IPv4) of the local host, otherwise "127.0.0.1". 
	 */
	private static String localHostIP = null;

	static {
		
		try {
			localHostIP = InetAddress.getLocalHost().getHostAddress();
		}
		catch (UnknownHostException u) {
			localHostIP = "127.0.0.1";
		}
	}

	/**
	 * Static variable containing the unique ID of the Common Logging Sample plug-in. 
	 */
    private final static String COMMON_LOGGING_SAMPLE_PLUGIN_ID = CommonLoggingSamplePlugin.getDefault().getBundle().getSymbolicName();
	
	/**
	 * Static variable containing the platform dependent line separator character. 
	 */
    private static final String LINE_SEPARATOR = System.getProperty("line.separator");

    
    /**
     * Run method for this sample class.
     * <p>
     * 
     * @param window The reference to the Eclipse workbench window.
     */
    public static void run(IWorkbenchWindow window) {

        try {

            //Scenario 1: Logging an <code>IStatus</code> log record to the sample plug-in's <code>ILog</code>:
            
            //Step A: Create an <code>IStatus</code> log record with one child <code>IStatus</code>:

            //Create the child <code>IStatus</code>:
            IStatus childStatus = new Status(IStatus.OK, COMMON_LOGGING_SAMPLE_PLUGIN_ID, IStatus.OK, "Child IStatus Message", new Throwable("Child Throwable Message"));
            
            //Create the <code>IStatus</code> log record:
            IStatus status = new MultiStatus(COMMON_LOGGING_SAMPLE_PLUGIN_ID, IStatus.OK,new IStatus[]{childStatus}, "IStatus Message", new Throwable("Throwable Message"));

            //Step B: Retrieve the sample plug-in's <code>ILog</code>:
            
            //Retrieve the sample plug-in's <code>ILog</code>:
            ILog logger = CommonLoggingSamplePlugin.getDefault().getLog();

            //Step C: Log the <code>IStatus</code> log record to the sample plug-in's <code>ILog</code> and notify the UI:
            logger.log(status);

            if(MessageDialogWithToggle.openQuestion(window.getShell(), "Common Logging Sample", "Scenario 1 (Logging an IStatus log record to the sample plug-in's ILog) successfully completed!" + LINE_SEPARATOR + LINE_SEPARATOR + "Continue?")){

                //Scenario 2: Logging an <code>ICommonBaseEventStatus</code> log record with a nested <code>CommonBaseEvent</code> to the sample plug-in's <code>ILog</code>:

                //Step A: Create an <code>ICommonBaseEventStatus</code> log record with a nested <code>CommonBaseEvent</code>:
    			ICommonBaseEventStatus commonBaseEventStatus = new CommonBaseEventStatus(createCommonBaseEvent("ICommonBaseEventStatus log record with a nested CommonBaseEvent logged to the sample plug-in's ILog"),COMMON_LOGGING_SAMPLE_PLUGIN_ID);
    			
                //Step B: Log the <code>ICommonBaseEventStatus</code> log record to the sample plug-in's <code>ILog</code> and notify the UI:
                logger.log(commonBaseEventStatus);

                if(MessageDialogWithToggle.openQuestion(window.getShell(), "Common Logging Sample", "Scenario 2 (Logging an ICommonBaseEventStatus log record to the sample plug-in's ILog) successfully completed!" + LINE_SEPARATOR + LINE_SEPARATOR + "Continue?")){
                
                    //Scenario 3: Logging a <code>CommonBaseEvent</code> log record, an <code>IStatus</code> log record with one child <code>IStatus</code> and an <code>ICommonBaseEventStatus</code> log record with a nested <code>CommonBaseEvent</code> to the sample plug-in's <code>ICommonBaseEventLog</code>:
                    
                    //Step A: Create the plug-in's <code>ICommonBaseEventLog</code>:

                    //Create the plug-in's <code>ICommonBaseEventLog</code>:
                    ICommonBaseEventLog commonBaseEventLog = new CommonBaseEventLog(CommonLoggingSamplePlugin.getDefault().getBundle());

                    //Step B: Log the <code>CommonBaseEvent</code> log record, an <code>IStatus</code> log record with one child <code>IStatus</code> and an <code>ICommonBaseEventStatus</code> log record with a nested <code>CommonBaseEvent</code> to the sample plug-in's <code>ICommonBaseEventLog</code> and notify the UI:
				    commonBaseEventLog.log(createCommonBaseEvent("CommonBaseEvent logged to the sample plug-in's ICommonBaseEventLog"));
				    commonBaseEventLog.log(status);
				    commonBaseEventLog.log(new CommonBaseEventStatus(createCommonBaseEvent("ICommonBaseEventStatus log record with a nested CommonBaseEvent logged to the sample plug-in's ICommonBaseEventLog"),COMMON_LOGGING_SAMPLE_PLUGIN_ID));
                    
                    if(MessageDialogWithToggle.openQuestion(window.getShell(), "Common Logging Sample", "Scenario 3 (Logging a CommonBaseEvent log record, an IStatus log record with one child IStatus and a ICommonBaseEventStatus log record with a nested CommonBaseEvent to the sample plug-in's ICommonBaseEventLog) successfully completed!" + LINE_SEPARATOR + LINE_SEPARATOR + "Continue?")){

                        //Scenario 4: Logging a <code>CommonBaseEvent</code> log record based on the sample plug-in's default Common Base Event template to the sample plug-in's <code>ICommonBaseEventLog</code>:

                        //Step A: Create a <code>CommonBaseEvent</code> log record based on the sample plug-in's default Common Base Event template:

            			//Retrieve the instance of the sample plug-in's <code>ICommonBaseEventLog</code> Event Factory:
            			EventFactory pluginsEventFactory = CommonLoggingPlugin.getEventFactory(CommonLoggingSamplePlugin.getDefault().getBundle());

            			//Create a new instance of a Common Base Event:
            			CommonBaseEvent configuredCommonBaseEvent = pluginsEventFactory.createCommonBaseEvent();

            			//Set the event's severity property with a fatal severity:
            			configuredCommonBaseEvent.setSeverity(CommonBaseEvent.SEVERITY_FATAL);
            			
            			//Set the subComponent property of the event's sourceComponentId property with the name of this class:
            			configuredCommonBaseEvent.getSourceComponentId().setSubComponent("CommonLoggingSample");

                        //Step B: Log the <code>CommonBaseEvent</code> to the sample plug-in's <code>ICommonBaseEventLog</code> and notify the UI:
            			commonBaseEventLog.log(configuredCommonBaseEvent);

                        if(MessageDialogWithToggle.openQuestion(window.getShell(), "Common Logging Sample", "Scenario 4 (Logging a CommonBaseEvent log record based on the sample plug-in's default Common Base Event template to the sample plug-in's ICommonBaseEventLog) successfully completed!" + LINE_SEPARATOR + LINE_SEPARATOR + "Continue?")){
                            
                            //Scenario 5: Logging a <code>CommonBaseEventLogRecord</code> log record with a nested <code>CommonBaseEvent</code> to the Common Logging logger (<code>java.util.logging.Logger</code>) named "Common.Logging.Logger":

        	                //Step A: Determine if the <code>CommonBaseEventLogRecord</code> log record with a nested <code>CommonBaseEvent</code> is loggable based on the sample plug-in's logging level and the <code>Level.SEVERE</code> logging level:
                        	if(CommonLoggingPlugin.isLoggable(COMMON_LOGGING_SAMPLE_PLUGIN_ID, Level.SEVERE)){
        	
                        		//Step B: Create the <code>CommonBaseEventLogRecord</code> log record with a nested <code>CommonBaseEvent</code>:                          
        		                CommonBaseEventLogRecord commonBaseEventLogRecord = new CommonBaseEventLogRecord(createCommonBaseEvent("CommonBaseEventLogRecord log record with a nested CommonBaseEvent logged to the Common Logging logger"));

                                //Step C: Set the <code>CommonBaseEventLogRecord</code> log record's <code>LoggerName</code> property to the Common Logging logger (<code>java.util.logging.Logger</code>) named "Common.Logging.Logger":
        		                commonBaseEventLogRecord.setLoggerName(CommonLoggingConstants.COMMON_LOGGING_JAVA_LOGGER_NAME);
        		
                                //Step D: Retrieve the Common Logging logger (<code>java.util.logging.Logger</code>) named "Common.Logging.Logger", log the <code>CommonBaseEventLogRecord</code> log record with a nested <code>CommonBaseEvent</code> and notify the UI:
        		                LogManager.getLogManager().getLogger(CommonLoggingConstants.COMMON_LOGGING_JAVA_LOGGER_NAME).log(commonBaseEventLogRecord);

                                MessageDialog.openInformation(window.getShell(), "Common Logging Sample", "Scenario 5 (Logging a CommonBaseEventLogRecord log record to the Common Logging logger (java.util.logging.Logger)) successfully completed!");
        	                }
                        }
                    }
                }                                         
            }

            MessageDialog.openInformation(window.getShell(), "Common Logging Sample", "Common Logging Sample successfully completed!" + LINE_SEPARATOR + LINE_SEPARATOR + "NOTE: When attaching to a Logging Agent, ensure logging is enabled in the Profiling and Logging preferences (Window >> Preferences >> Profiling and Logging >> Enable logging) and the Agent Controller service or daemon is running.");
        } 
        catch (Exception e) {
            MessageDialog.openError(window.getShell(), "Common Logging Sample", "Common Logging Sample error:" + LINE_SEPARATOR + e);
        }
    }
    
    private static CommonBaseEvent createCommonBaseEvent(String message){
		
    	//Retrieve the instance of the Event Factory:
		EventFactory eventFactory = EventFactoryContext.getInstance().getSimpleEventFactoryHome().getAnonymousEventFactory();

		//Create a new instance of a Report Situation:
		ReportSituation reportSituation = eventFactory.createReportSituation();
		reportSituation.setReasoningScope("INTERNAL");
		reportSituation.setReportCategory("LOG");

		//Create a new instance of a Situation:
		Situation situation = eventFactory.createSituation();
		situation.setCategoryName("ReportSituation");
		situation.setSituationType(reportSituation);

		//Create a new instance of a Source Component:
		ComponentIdentification sourceComponentId = eventFactory.createComponentIdentification();
		sourceComponentId.setExecutionEnvironment("Java");
		sourceComponentId.setApplication("Common Logging");
		sourceComponentId.setComponent("common.logging.sample.plugin");
		sourceComponentId.setSubComponent("CommonLoggingSample");
		sourceComponentId.setComponentIdType(ComponentIdentification.COMPONENT_ID_TYPE_APPLICATION);
		sourceComponentId.setComponentType("Eclipse Plug-ins");    		
		sourceComponentId.setLocationType(ComponentIdentification.LOCATION_TYPE_IPV4);
		sourceComponentId.setLocation(localHostIP);
		sourceComponentId.setThreadId(Thread.currentThread().getName());

		//Create a new instance of a <code>CommonBaseEvent</code>:
		CommonBaseEvent commonBaseEvent = eventFactory.createCommonBaseEvent();
		
		//Set the event's globalInstanceId property with a new GUID:
        commonBaseEvent.setGlobalInstanceId(Guid.generate());
		
		//Set the event's sourceComponentId property with created Source Component:
        commonBaseEvent.setSourceComponentId(sourceComponentId);

		//Set the event's situation property with created Situation:
        commonBaseEvent.setSituation(situation);
		
		//Set the event's creationTime property with the current time stamp:
        commonBaseEvent.setCreationTimeAsLong(System.currentTimeMillis());

		//Set the event's severity property with a fatal severity:
        commonBaseEvent.setSeverity(CommonBaseEvent.SEVERITY_FATAL);
		
		//Set the event's msg property with a sample message:
        commonBaseEvent.setMsg(message);
        
        //Add an <code>ExtendedDataElement</code> with a stack trace to the event:
        commonBaseEvent.addExtendedDataElement(EventHelpers.convertToExtendedDataElement(new Exception()));

        return commonBaseEvent;
    }
}